#include "stdafx.h"
#include "ListShares.h"
#include "ListSharesDlg.h"
#include "HyperLink.h"
#include "StrConv.h"
#include <lm.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

typedef struct _SHAREINFO
{
	CString strName;
	CString strPath;
	CString strComment;
	DWORD dwType;
	DWORD dwUsers;
} SHAREINFO, *PSHAREINFO;

#ifndef LVM_GETHEADER
#define LVM_GETHEADER	(LVM_FIRST + 31)
#endif

#ifndef HDM_SETIMAGELIST
#define HDM_SETIMAGELIST	(HDM_FIRST + 8)
#endif

//***********************************************
inline bool Is256ColorSupported(void)
{
	bool bResult = false;

	// Return true if screen deivce supports 256 colors or better
	HDC hdc = ::GetDC(NULL);
	if(hdc != NULL)
	{
		if(::GetDeviceCaps(hdc, BITSPIXEL) >= 8)
			bResult = true;
		::ReleaseDC(NULL, hdc);
	}

	return bResult;
}

//***********************************************
void DialogEnableWindow(CDialog *pDlg, CWnd *pWnd, const bool bEnable)
{
	_ASSERTE(pDlg);
	_ASSERTE(pWnd);

	// If we're disabling a control with focus, move focus to the next control first
	if(!bEnable && pWnd == CWnd::GetFocus())
		pDlg->NextDlgCtrl();

	// Disable the control
	pWnd->EnableWindow(bEnable);

	// If we enabled a default button, reset it as the default button (this forces the UI to repaint the darker frame)
	LRESULT n = pDlg->GetDefID();
	if(bEnable && HIWORD(n) && LOWORD(n) == LOWORD(pWnd->GetDlgCtrlID()))
		pDlg->SetDefID(LOWORD(n));
}

//***********************************************
inline void DialogSetFocus(CDialog *pDlg, CWnd *pWnd)
{
	_ASSERTE(pDlg);
	_ASSERTE(pWnd);

	pDlg->GotoDlgCtrl(pWnd);
}

//***********************************************
class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CHyperLink	m_ctrlHyperLink;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange *pDX);
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//***********************************************
CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

//***********************************************
void CAboutDlg::DoDataExchange(CDataExchange *pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Control(pDX, IDC_STATIC_URL, m_ctrlHyperLink);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//***********************************************
CListSharesDlg::CListSharesDlg(CWnd *pParent)
	: CDialog(CListSharesDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CListSharesDlg)
	//}}AFX_DATA_INIT

	// Load the icons and cursors
	m_hIconLarge = (HICON)LoadImage(AfxGetInstanceHandle(),
		MAKEINTRESOURCE(IDR_MAINFRAME),
		IMAGE_ICON,
		GetSystemMetrics(SM_CXICON),
		GetSystemMetrics(SM_CYICON),
		0);
	m_hIconSmall = (HICON)LoadImage(AfxGetInstanceHandle(),
		MAKEINTRESOURCE(IDR_MAINFRAME),
		IMAGE_ICON,
		GetSystemMetrics(SM_CXSMICON),
		GetSystemMetrics(SM_CYSMICON),
		0);

	m_bSortAscending = true;
	m_nSortColumn = 0;

	// Get the local machine's name
	try
	{
		PWKSTA_INFO_100 pwi100 = NULL;
		NET_API_STATUS status = NetWkstaGetInfo(NULL, 100, (PBYTE*)&pwi100);
		if(status == NERR_Success)
		{
			PTSTR szLocalMachine = Wide2Text((PWSTR)pwi100->wki100_computername);
			m_strLocalMachine = szLocalMachine;
			delete[] szLocalMachine;
			NetApiBufferFree(pwi100);
		}
	}
	catch(...) {}
}

//***********************************************
CListSharesDlg::~CListSharesDlg()
{
	if(m_hIconLarge != NULL) DestroyIcon(m_hIconLarge);
	if(m_hIconSmall != NULL) DestroyIcon(m_hIconSmall);
}

//***********************************************
void CListSharesDlg::DoDataExchange(CDataExchange *pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CListSharesDlg)
	DDX_Control(pDX, IDC_LIST_SHARES, m_ctrlShares);
	DDX_Text(pDX, IDC_EDIT_MACHINE, m_strMachine);
	DDV_MaxChars(pDX, m_strMachine, 20);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CListSharesDlg, CDialog)
	//{{AFX_MSG_MAP(CListSharesDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_SYSCOLORCHANGE()
	ON_BN_CLICKED(IDC_BUTTON_REFRESH, OnRefresh)
	ON_NOTIFY(LVN_COLUMNCLICK, IDC_LIST_SHARES, OnColumnClickListShares)
	ON_EN_CHANGE(IDC_EDIT_MACHINE, OnChangeMachine)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//***********************************************
BOOL CListSharesDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// IDM_ABOUTBOX must be in the system command range.
	_ASSERTE((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	_ASSERTE(IDM_ABOUTBOX < 0xF000);

	CMenu *pSysMenu = GetSystemMenu(FALSE);
	if(pSysMenu != NULL)
	{
		// Remove the "Size" and "Maximize" menu options from the system menu
		pSysMenu->DeleteMenu(SC_SIZE, MF_BYCOMMAND);
		pSysMenu->DeleteMenu(SC_MAXIMIZE, MF_BYCOMMAND);

		// Append a separator and About
		CString strMenu;
		pSysMenu->AppendMenu(MF_SEPARATOR);
		strMenu.LoadString(IDS_ABOUTBOX); _ASSERTE(!strMenu.IsEmpty());
		pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strMenu);
	}

	SetIcon(m_hIconLarge, TRUE);		// Set big icon
	SetIcon(m_hIconSmall, FALSE);		// Set small icon

	m_strMachine = m_strLocalMachine;
	UpdateData(FALSE);

	InitSharesList();
	FillSharesList();
	OnChangeMachine();

	DialogSetFocus(this, GetDlgItem(IDC_EDIT_MACHINE));
	return FALSE;
}

//***********************************************
BOOL CListSharesDlg::DestroyWindow()
{
	ClearSharesList();

	return CDialog::DestroyWindow();
}

//***********************************************
void CListSharesDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

//***********************************************
void CListSharesDlg::OnPaint()
{
	if(IsIconic())
	{
		CPaintDC dc(this);

		SendMessage(WM_ICONERASEBKGND, (WPARAM)dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIconLarge);
	}
	else
	{
		CDialog::OnPaint();
	}
}

//***********************************************
HCURSOR CListSharesDlg::OnQueryDragIcon()
{
	return (HCURSOR)m_hIconSmall;
}

//***********************************************
void CListSharesDlg::OnSysColorChange()
{
	CDialog::OnSysColorChange();
	SetColSortIndicator();
}

//***********************************************
void CListSharesDlg::InitSharesList(void)
{
	_ASSERTE(IsWindow(m_ctrlShares.m_hWnd));

#if 1
	m_ctrlShares.SendMessage(LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT);
#else
	m_ctrlShares.SetExtendedStyle(m_ctrlShares.GetExtendedStyle() | LVS_EX_FULLROWSELECT);
#endif

	// Set the image list
	UINT nBmpID = IDB_SMALLICONS, nFlags = ILC_COLOR16 | ILC_MASK;
	if(!Is256ColorSupported())
	{
		nBmpID = IDB_SMALLICONS16;
		nFlags = ILC_COLOR4 | ILC_MASK;
	}

	CImageList ilSmall;
	ilSmall.Create(16, 16, nFlags, 0, 2);
	CBitmap bmpSmallIcons; VERIFY(bmpSmallIcons.LoadBitmap(nBmpID));
	ilSmall.Add(&bmpSmallIcons, RGB(255, 0, 255));
	ListView_SetImageList(m_ctrlShares.GetSafeHwnd(), ilSmall.Detach(), LVSIL_SMALL);

	// Create column
	CRect rect;
	m_ctrlShares.GetClientRect(&rect);

	CString strColumn;
	int nCol1Width, nCol2Width, nCol3Width, nCol4Width, nCol5Width;
	int nScrollBarWidth = GetSystemMetrics(SM_CXVSCROLL);

	// Size the columns
	CDC *pDC = m_ctrlShares.GetWindowDC();
	strColumn.LoadString(IDS_COLUMN_USERS);
	CSize textsize = pDC->GetTextExtent(strColumn);
	m_ctrlShares.ReleaseDC(pDC);
	nCol3Width = textsize.cx;

	nCol4Width = nCol5Width = 130;
	nCol4Width = nCol5Width = 90;
	nCol2Width = (rect.Width() - nCol3Width - nCol4Width - nCol5Width - nScrollBarWidth) / 2;
	nCol1Width = rect.Width() - nCol2Width - nCol3Width - nCol4Width - nCol5Width - nScrollBarWidth;

	strColumn.LoadString(IDS_COLUMN_NAME);
	m_ctrlShares.InsertColumn(0, strColumn, LVCFMT_LEFT, nCol1Width, 0);
	strColumn.LoadString(IDS_COLUMN_TYPE);
	m_ctrlShares.InsertColumn(1, strColumn, LVCFMT_LEFT, nCol2Width, 1);
	strColumn.LoadString(IDS_COLUMN_USERS);
	m_ctrlShares.InsertColumn(2, strColumn, LVCFMT_RIGHT, nCol3Width, 2);
	strColumn.LoadString(IDS_COLUMN_PATH);
	m_ctrlShares.InsertColumn(3, strColumn, LVCFMT_LEFT, nCol4Width, 3);
	strColumn.LoadString(IDS_COLUMN_COMMENT);
	m_ctrlShares.InsertColumn(4, strColumn, LVCFMT_LEFT, nCol5Width, 4);

	SetColSortIndicator();
}

//***********************************************
void CListSharesDlg::FillSharesList(void)
{
	try
	{
		if(m_ctrlShares.IsWindowVisible())
			m_ctrlShares.SetRedraw(FALSE);
		ClearSharesList();

		CString str;
		DWORD dw, dwRead, dwTotal, dwResume = 0;
		int nIndex;
		PSHARE_INFO_2 psi2 = NULL;
		NET_API_STATUS status;
		PSHAREINFO psi = NULL;
		TCHAR szText[_MAX_PATH];

		WCHAR wszMachine[64];
		Text2Wide(m_strMachine, wszMachine);

		LV_ITEM lvi = {0};
		lvi.mask = LVIF_TEXT | LVIF_IMAGE;

		do{
			status = NetShareEnum((PTSTR)wszMachine,
				2,
				(PBYTE*)&psi2,
				MAX_PREFERRED_LENGTH,
				&dwRead,
				&dwTotal,
				&dwResume);

			if(status == NERR_Success || status == ERROR_MORE_DATA)
			{
				for(dw = 0; dw < dwRead; dw++)
				{
					// Get tjhe icon for the share
					switch(psi2[dw].shi2_type)
					{
					case STYPE_DISKTREE:
						lvi.iImage = 0;
						break;
					case STYPE_SPECIAL | STYPE_DISKTREE:
						lvi.iImage = 1;
						break;
					case STYPE_PRINTQ:
						lvi.iImage = 2;
						break;
					case STYPE_SPECIAL | STYPE_PRINTQ:
						lvi.iImage = 3;
						break;
					case STYPE_DEVICE:
						lvi.iImage = 4;
						break;
					case STYPE_SPECIAL | STYPE_DEVICE:
						lvi.iImage = 5;
						break;
					case STYPE_IPC:
						lvi.iImage = 6;
						break;
					case STYPE_SPECIAL | STYPE_IPC:
						lvi.iImage = 7;
						break;
					default:
						lvi.iImage = 8;
						break;
					}

					lvi.pszText = Wide2Text((PWSTR)psi2[dw].shi2_netname);
					psi = new SHAREINFO;
					psi->strName = lvi.pszText;
					psi->dwType = psi2[dw].shi2_type;
					psi->dwUsers = psi2[dw].shi2_current_uses;

					nIndex = m_ctrlShares.InsertItem(&lvi);

					str.Format(_T("%d"), psi2[dw].shi2_current_uses);
					Wide2Text((PWSTR)psi2[dw].shi2_path, szText);
					psi->strPath = szText;
					Wide2Text((PWSTR)psi2[dw].shi2_remark, szText);
					psi->strComment = szText;

					m_ctrlShares.SetItemText(nIndex, 1, ShareTypeToString(psi->dwType));
					m_ctrlShares.SetItemText(nIndex, 2, str);
					m_ctrlShares.SetItemText(nIndex, 3, psi->strPath);
					m_ctrlShares.SetItemText(nIndex, 4, psi->strComment);
					m_ctrlShares.SetItemData(nIndex, (DWORD)psi);

					lvi.iItem++;
					delete[] lvi.pszText; lvi.pszText = NULL;
				}

				NetApiBufferFree(psi2);
			}
		}while(status == ERROR_MORE_DATA);

		m_ctrlShares.SortItems(SortFunc, (DWORD)this);
	}
	catch(...) {}

	if(m_ctrlShares.IsWindowVisible())
	{
		m_ctrlShares.SetRedraw();
		m_ctrlShares.InvalidateRect(NULL);
	}
}

//***********************************************
void CListSharesDlg::ClearSharesList(void)
{
	PSHAREINFO psi = NULL;
	for(int nIndex = 0; nIndex < m_ctrlShares.GetItemCount(); nIndex++)
	{
		psi = (PSHAREINFO)m_ctrlShares.GetItemData(nIndex);
		delete psi;
	}

	m_ctrlShares.DeleteAllItems();
}

//***********************************************
void CListSharesDlg::OnRefresh()
{
	CWaitCursor wait;
	UpdateData();
	FillSharesList();
}

//***********************************************
void CListSharesDlg::OnColumnClickListShares(NMHDR *pNMHDR, LRESULT *pResult)
{
	NM_LISTVIEW *pNMListView = (NM_LISTVIEW*)pNMHDR;

	_ASSERTE(pNMListView->iItem == -1);

	SetColSortIndicator(false);

	if(pNMListView->iSubItem == m_nSortColumn)
	{
		m_bSortAscending ^= true;
	}
	else
	{
		m_bSortAscending = true;
		m_nSortColumn = pNMListView->iSubItem;
	}

	m_ctrlShares.SortItems(SortFunc, (DWORD)this);
	SetColSortIndicator();

	*pResult = 0;
}

//***********************************************
int CALLBACK CListSharesDlg::SortFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	_ASSERTE(lParam1);
	_ASSERTE(lParam2);
	_ASSERTE(lParamSort);

	CListSharesDlg *pThis = (CListSharesDlg*)lParamSort;
	PSHAREINFO psi1 = (PSHAREINFO)lParam1;
	PSHAREINFO psi2 = (PSHAREINFO)lParam2;

	int nSortRtn = 0;

	if(pThis->m_nSortColumn == 0)
	{
		nSortRtn = psi1->strName.CompareNoCase(psi2->strName);
	}
	else if(pThis->m_nSortColumn == 1)
	{
		if(psi1->dwType > psi2->dwType)
			nSortRtn = 1;
		else if(psi1->dwType < psi2->dwType)
			nSortRtn = -1;
	}
	else if(pThis->m_nSortColumn == 2)
	{
		if(psi1->dwUsers > psi2->dwUsers)
			nSortRtn = 1;
		else if(psi1->dwUsers < psi2->dwUsers)
			nSortRtn = -1;
	}
	else if(pThis->m_nSortColumn == 3)
	{
		nSortRtn = psi1->strPath.CompareNoCase(psi2->strPath);
	}
	else if(pThis->m_nSortColumn == 4)
	{
		nSortRtn = psi1->strComment.CompareNoCase(psi2->strComment);
	}

	if(!pThis->m_bSortAscending)
		nSortRtn = -nSortRtn;

	return nSortRtn;
}

//***********************************************
void CListSharesDlg::SetColSortIndicator(const bool bEnable)
{
	HDITEM hdi = {0};
	hdi.mask = HDI_FORMAT | HDI_BITMAP;
#if 1
	HWND hWndHeader = (HWND)::SendMessage(m_ctrlShares.GetSafeHwnd(), LVM_GETHEADER, 0, 0);
	CHeaderCtrl *pHeaderCtrl = (CHeaderCtrl*)CHeaderCtrl::FromHandle(hWndHeader);
#else
	CHeaderCtrl *pHeaderCtrl = m_ctrlShares.GetHeaderCtrl();
#endif
	pHeaderCtrl->GetItem(m_nSortColumn, &hdi);
	if(hdi.hbm)
	{
		DeleteObject(hdi.hbm);
		hdi.hbm = NULL;
	}

	if(bEnable)
	{
		hdi.fmt |= HDF_BITMAP | HDF_BITMAP_ON_RIGHT;
		hdi.hbm = (HBITMAP)LoadImage(AfxGetInstanceHandle(), MAKEINTRESOURCE(m_bSortAscending ? IDB_UP : IDB_DOWN), IMAGE_BITMAP, 0, 0, LR_LOADMAP3DCOLORS);
		pHeaderCtrl->SetItem(m_nSortColumn, &hdi);
    }
	else
	{
		hdi.fmt &= ~(HDF_BITMAP | HDF_BITMAP_ON_RIGHT);
		pHeaderCtrl->SetItem(m_nSortColumn, &hdi);
	}
}

//***********************************************
CString CListSharesDlg::ShareTypeToString(const DWORD dwType)
{
	CString strType;

	switch(dwType)
	{
	case STYPE_DISKTREE:
	case STYPE_SPECIAL | STYPE_DISKTREE:
		strType = _T("Disk Drive");
		break;
	case STYPE_PRINTQ:
	case STYPE_SPECIAL | STYPE_PRINTQ:
		strType = _T("Print Queue");
		break;
	case STYPE_DEVICE:
	case STYPE_SPECIAL | STYPE_DEVICE:
		strType = _T("Communication Device");
		break;
	case STYPE_IPC:
	case STYPE_SPECIAL | STYPE_IPC:
		strType = _T("Interprocess Communication");
		break;
	default:
		strType = _T("Unknown");
	}

	return strType;
}

//***********************************************
void CListSharesDlg::OnChangeMachine()
{
	UpdateData();
	DialogEnableWindow(this, GetDlgItem(IDC_BUTTON_REFRESH), !m_strMachine.IsEmpty());
}
